@@ -200,7 +200,7 @@
       }
       else if ((nchar(im <- Sys.which("convert"))>0)&& ##
                      (toupper(normalizePath(dirname(dirname(im))))!=
-                      toupper(normalizePath(Sys.getenv("WINDIR"))))) {
+                      toupper(normalizePath(Sys.getenv("WINDIR"),mustWork=FALSE)))) {
         # print("convert is not in Windows directory")
          cmd <- paste(im,fileout,ifelse(n>=255,"-colors 255","")
                      ,paste0("png8:",fileout))
